from .adaboost_classifier import AdaBoostClassifierContainer
from .catboost_classifier import CatBoostClassifierContainer
from .decisiontree_classifier import DecisionTreeClassifierContainer
from .extratrees_classifier import ExtraTreesClassifierContainer
from .gaussianNB_classifier import GaussianNBContainer
from .gradient_boosting_classifier import GradientBoostingClassifierContainer
from .kneighbors_classifier import KNeighborsClassifierContainer
from .lgbm_classifier import LGBMClassifierContainer
from .lineardiscriminantanalysis import LinearDiscriminantAnalysisContainer
from .logisticregression import LogisticRegressionContainer
from .mlpclassifier import MLPClassifierContainer
from .quadraticdiscriminantanalysis import QuadraticDiscriminantAnalysisContainer
from .randomforest_classifier import RandomForestClassifierContainer
# from .ridge_classifier import RidgeClassifierContainer
# from .sgd_classifier import SGDClassifierContainer
# from .svc_classifier import SVCClassifierContainer
from .xgb_classifier import XGBClassifierContainer
